#include "call.h"
#include <stdio.h>

int dprintf(char* fmt, ...)
{
	char printString[5000];
	va_list argp;
	va_start(argp, fmt);
	vsprintf(printString, fmt, argp);
	va_end(argp);
    
	OutputDebugString(printString);
	return strlen(printString);
}

Call::Call(AVSValue args, IScriptEnvironment* env) : GenericVideoFilter(args[0].AsClip())
{
	m_nFrameCount = 0;
	strcpy(m_szCommand, args[1].AsString());
	char *szFrames = (char*) args[2].AsString();

	memset(m_bDoneFrame, 0, MAX_FRAMES);
	m_bDoneLast = false;

	// Get the string of frames to call the app on
	char szTemp[50];
	char *p = szTemp;
	int len = strlen(szFrames);
	for ( int i = 0; i <= len; i++ )
	{
		*(p++) = szFrames[i];
		if ( szFrames[i] == ',' || szFrames[i] == ';' || szFrames[i] == 0 )
		{
			*(p-1) = 0;
			m_nFrames[m_nFrameCount] = atoi(szTemp);
			p = szTemp;
			
			dprintf("Going to do on frame: %d \n", m_nFrames[m_nFrameCount]);
			m_nFrameCount++;
		}
	}

	len = strlen(m_szCommand);
	for ( i = 0; i <= len; i++ )
	{
		if ( m_szCommand[i] == '\'' )
			m_szCommand[i] = '\"';
	}

	dprintf("Command: %s", m_szCommand);

	// If -2 has been specified do the do in the constructor
	if ( CheckDoNow(-2, 1) )
	{
		STARTUPINFO stinfo;
		PROCESS_INFORMATION pinfo;

		ZeroMemory(&stinfo, sizeof(STARTUPINFO));
		ZeroMemory(&pinfo, sizeof(PROCESS_INFORMATION));

		stinfo.cb = sizeof(STARTUPINFO);

		BOOL bOK = CreateProcess(	NULL, 
									m_szCommand, 
									NULL, 
									NULL, 
									FALSE, 
									0, 
									NULL, 
									NULL, 
									&stinfo,
									&pinfo);

		if ( bOK )
			WaitForSingleObject(pinfo.hThread, INFINITE);
	}
}

bool Call::CheckDoNow(int n, int max)
{
	for ( int i = 0; i != m_nFrameCount; i++ )
	{
		if ( n == m_nFrames[i] && !m_bDoneFrame[i] )
		{
			m_bDoneFrame[i] = true;
			return true;
		}

		if ( n == max && m_nFrames[i] == -1 && !m_bDoneLast )
		{
			m_bDoneLast = true;
			return true;
		}
	}

	return false;
}

PVideoFrame __stdcall Call::GetFrame(int n, IScriptEnvironment* env)
{
	PVideoFrame cf = child->GetFrame(n, env);

	const VideoInfo vf = GetVideoInfo();

	// vf.num_frames-1 because otherwise not called ??
	if ( CheckDoNow(n, vf.num_frames-1) ) 
	{
		dprintf("Doing command on frame: %d \n", n);
		STARTUPINFO stinfo;
		PROCESS_INFORMATION pinfo;

		ZeroMemory(&stinfo, sizeof(STARTUPINFO));
		ZeroMemory(&pinfo, sizeof(PROCESS_INFORMATION));

		stinfo.cb = sizeof(STARTUPINFO);

		BOOL bOK = CreateProcess(	NULL, 
									m_szCommand, 
									NULL, 
									NULL, 
									FALSE, 
									0, 
									NULL, 
									NULL, 
									&stinfo,
									&pinfo);

		if ( bOK )
			WaitForSingleObject(pinfo.hThread, INFINITE);
	}

	return cf;
}

AVSValue __cdecl Create_Call(AVSValue args, void*, IScriptEnvironment* env)
{
  return new Call(args, env);
}


extern "C" __declspec(dllexport) const char* __stdcall 
#ifdef YV12_VERSION
	AvisynthPluginInit2(IScriptEnvironment* env)
#else
	AvisynthPluginInit(IScriptEnvironment* env)
#endif
{
  env->AddFunction("Call", "c[t]s[a]s", Create_Call, 0);
  return "`Create_Call'";
}


BOOL WINAPI DllMain(
  HINSTANCE hinstDLL,  // handle to the DLL module
  DWORD fdwReason,     // reason for calling function
  LPVOID lpvReserved   // reserved
)
{
	return TRUE;
}


// For testing purposes....of course ;)
int WINAPI WinMain(
  HINSTANCE hInstance,      // handle to current instance
  HINSTANCE hPrevInstance,  // handle to previous instance
  LPSTR lpCmdLine,          // command line
  int nCmdShow              // show state
)
{
	return 0;
}

